/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2021 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

#ifndef XCSOAR_TEXT_ROW_RENDERER_HPP
#define XCSOAR_TEXT_ROW_RENDERER_HPP

#include <tchar.h>

struct PixelRect;
class Font;
class Canvas;

/**
 * A helper for drawing a text row into a rectangular area.
 */
class TextRowRenderer {
  unsigned left_padding, top_padding;

public:
  /**
   * @return the row height (including top and bottom padding)
   */
  unsigned CalculateLayout(const Font &font) noexcept;

  void DrawTextRow(Canvas &canvas, const PixelRect &rc,
                   const TCHAR *text) const noexcept;

  /**
   * Returns the minimum X coordinate of the column after the given
   * text.
   */
  [[gnu::pure]]
  int NextColumn(Canvas &canvas, const PixelRect &rc,
                 const TCHAR *text) const noexcept;

  /**
   * Combine DrawTextRow() and NextColumn().
   */
  int DrawColumn(Canvas &canvas, const PixelRect &rc,
                 const TCHAR *text) const noexcept;

  /**
   * Returns the maximum X coordinate of the column before the given
   * text.
   */
  [[gnu::pure]]
  int PreviousRightColumn(Canvas &canvas, const PixelRect &rc,
                          const TCHAR *text) const noexcept;

  /**
   * Draws a right-aligned column and returns the new "right"
   * coordinate.
   */
  int DrawRightColumn(Canvas &canvas, const PixelRect &rc,
                      const TCHAR *text) const noexcept;
};

#endif
